import itertools
from .offer import Offer
from bs4 import BeautifulSoup
from urllib.parse import urlencode


def _html_to_soup(response):
    """Convert web response to BeautifulSoup object"""
    return BeautifulSoup(response.text, "html.parser")


def _get_offer_list(id, base_market_url):
    """Generates URL for a particular Character Id"""
    int_id = int(id)
    return base_market_url + urlencode({'action': 'buy', 'page': 0,
                                        'id_familly': 0, 'sortby': 'price',
                                        'orderby': 'asc', 'rarity': 'all',
                                        'group': 'all', 'id_artist': 0,
                                        'id_format': 0, 'search': int_id})


def _find_offers(page):
    """Finds all relevant offers for a given BeautifulSoup page"""
    return Offer(_get_offer_id(page), _create_offer_dict(page))


def _get_offer_id(page):
    """Gets the relevant Character Id for the BeautifulSoup page"""
    return int(page.find(class_="text-character")['data-character-id'])


def _create_offer_dict(page):
    """Matches prices to star level to return dict of lowest prices per level"""
    return {
        len(row.find_all(src='https://s.acdn.ur-img.com/img/v3/card/icon-star-on.png')):
            "".join(next(itertools.islice(row.stripped_strings, 2, 3)).split())
        for row in reversed(page.tbody.find_all('tr'))
    }



class Offer:
    @property
    def id(self):
        """
        The character id this offer refers to.

        :getter: Returns this Offer's id
        :setter: Sets this Offer's id
        :type: int (conversion is applied)
        """
        return self.__id

    @id.setter
    def id(self, id):
        self.__id = int(id)

    @property
    def level_price_dict(self):
        """
        The dictionary of level-to-price relationships for \
        this offer.

        :getter: Returns this Offer's level-price-dict
        :setter: Sets this Offer's level-price-dict
        :type: dict (sets to None if not type conformant)
        """
        return self.__level_price_dict

    @level_price_dict.setter
    def level_price_dict(self, lpd):
        self.__level_price_dict = lpd if isinstance(lpd, dict) else None
        if self.level_price_dict is not None and not self._is_level_price_dict_sorted():
            self._sort_level_price_dict()

    def __init__(self, id=0, level_price_dict=None):
        self.id = id
        self.level_price_dict = level_price_dict

    def _sort_level_price_dict(self):
        """Sorts dict with lowest price first"""
        if self.level_price_dict is not None:
            self.level_price_dict = {k: v for k, v in sorted(self.level_price_dict.items(),
                                                             key=lambda item: item[1])}

    def _is_level_price_dict_sorted(self):
        """Checks if dict is already sorted"""
        values = list(self.level_price_dict.values())
        return all(values[i] <= values[i + 1] for i in range(len(values) - 1))

    def get_min_price(self):
        """Return lowest price in dict"""
        return self.level_price_dict[self.get_min_level()]

    def get_min_level(self):
        """Return level for lowest price in dict"""
        return next(iter(self.level_price_dict))

    def get_rel_price(self, target_level):
        """Return price of target level, or min price if not available"""
        return self.level_price_dict.get(int(target_level), self.get_min_price())

    def get_rel_level(self, target_level):
        """Return target level, or min level if not available"""
        return target_level if int(target_level) in self.level_price_dict \
            else self.get_min_level()


from .offer_list import _html_to_soup, _get_offer_list, _find_offers


#Constants
BASE_MARKET_URL = "https://www.urban-rivals.com/market/?"


def _clean_input(id):
    """Cleans the char id"""
    if type(id) is int:
        id = abs(id)
    return id


def get_market_offers(session, ids, base_market_url=BASE_MARKET_URL):
    """\nMain function for interaction with this library.
    \nProvided a sequence of Character Ids, returns a dictionary of offers for each. \
    Requires a session which has already authenticated with Urban Rivals.
    \nOptional: provide a base market URL for proxy. Must end with a "?" \
    Ex: "http://example.com?"


    >>>get_market_offers(session, [1400, 1423, 1764])

    {1400: Offer, 1423: Offer, 1764: Offer}


    >>>get_market_offers(session, ["1301", "1543"])

    {"1301": Offer, "1543": Offer}

    """
    if len(ids) < 1:
        raise ValueError("Ids cannot be empty")
    if not base_market_url.endswith("?"):
        raise ValueError("URL must end with a question mark")
    market = {
        char_id:
        _html_to_soup(
            session.get(
                _get_offer_list(char_id, base_market_url)
        ))
        for char_id in map(_clean_input, ids)
    }
    return {char_id :_find_offers(market[char_id])
            for char_id in map(_clean_input, ids) }


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['urmarketscraper']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2', 'requests>=2.23.0,<3.0.0']

setup_kwargs = {
    'name': 'urmarketscraper',
    'version': '1.0.0',
    'description': 'Collects market offers for Urban Rivals.',
    'long_description': '#Urban Rivals Market Scraper\n\n##Collects market offers for Urban Rivals.\n\nThis is a screen scraper utility for [Urban Rivals](https://www.urban-rivals.com).\n\nTo use, call `offer_list.get_market_offers`\n\nThe first parameter is the `requests.Session()` object which contains a logged-in\nuser. The second parameter is the list of `ids` for the characters in question.\nThe third parameter is an optional override for what what the base market address\nis. Default is `https://www.urban-rivals.com/market/?`. Any url given here must end\nwith a `?` so that the URL encoding can complete correctly.\n\n[Basic Usage Example](basic_usage.txt)',
    'author': 'Brent Spector',
    'author_email': 'brent.spector@yahoo.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/brentspector/urmarketscraper',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
